use image::Rgba;

#[derive(Clone, Copy)]
pub struct ColourMap {
    background: Rgba<u8>,
    foreground: Rgba<u8>,
}

impl ColourMap {
    pub(crate) fn from(palette: &bitsy_parser::Palette) -> ColourMap {
        let background = Rgba::from([
            palette.colours[0].red,
            palette.colours[0].green,
            palette.colours[0].blue,
            255,
        ]);

        let foreground = Rgba::from([
            palette.colours[1].red,
            palette.colours[1].green,
            palette.colours[1].blue,
            255,
        ]);

        ColourMap { background, foreground }
    }
}

fn diff(a: &Rgba<u8>, b:&Rgba<u8>) -> u32 {
    let diff_red  = (a[0] as i16 - b[0] as i16).abs();
    let diff_green= (a[1] as i16 - b[1] as i16).abs();
    let diff_blue = (a[2] as i16 - b[2] as i16).abs();

    (diff_red + diff_green + diff_blue) as u32
}

impl image::imageops::colorops::ColorMap for ColourMap {
    type Color = Rgba<u8>;

    #[inline(always)]
    fn index_of(&self, color: &Self::Color) -> usize {
        let diff_background = diff(color, &self.background);
        let diff_foreground = diff(color, &self.foreground);

        if diff_foreground <= diff_background { 1 } else { 0 }
    }

    #[inline(always)]
    fn lookup(&self, idx: usize) -> Option<Self::Color> {
        match idx {
            0 => Some(self.background.into()),
            1 => Some(self.foreground.into()),
            _ => None,
        }
    }

    /// Indicate NeuQuant implements `lookup`.
    fn has_lookup(&self) -> bool {
        true
    }

    #[inline(always)]
    fn map_color(&self, color: &mut Self::Color) {
        let closest = match self.index_of(color) {
            1 => self.foreground,
            _ => self.background,
        };

        color[0] = closest[0];
        color[1] = closest[1];
        color[2] = closest[2];
        color[3] = closest[3];
    }
}
